﻿using System;
using System.IO;
using System.Globalization;
using System.Text;
using System.Xml;
using MathEx.ExternalBallistic.Units;
using MathEx.ExternalBallistic.JBM;

namespace MathEx.ExternalBallistic
{
    public static class AmmoInfoController
    {
        public static class Serialization
        {
            public static void Write(XmlDocument doc, XmlNode parent, AmmoInfo ammo)
            {
                XmlNode node = doc.CreateElement("ammo-info");
                XmlAttribute attr;
                attr = doc.CreateAttribute("table");
                attr.Value = DragTableFactory.DragTableName(ammo.Table);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("bc");
                attr.Value = ammo.BallisticCoefficient.ToString("0.000", CultureInfo.InvariantCulture);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("bullet-weight");
                attr.Value = ammo.BulletWeight.ToString(ammo.BulletWeight.SetUnit, false, 5);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("muzzle-velocity");
                attr.Value = ammo.MuzzleVelocity.ToString(ammo.MuzzleVelocity.SetUnit, false, 5);
                node.Attributes.Append(attr);
                parent.AppendChild(node);
            }

            public static void Write(XmlDocument doc, XmlNode parent, AmmoInfoEx ammo)
            {
                XmlNode node = doc.CreateElement("ammo-info-ex");
                XmlAttribute attr;
                attr = doc.CreateAttribute("table");
                attr.Value = DragTableFactory.DragTableName(ammo.Table);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("bc");
                attr.Value = ammo.BallisticCoefficient.ToString("0.000", CultureInfo.InvariantCulture);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("bullet-weight");
                attr.Value = ammo.BulletWeight.ToString(ammo.BulletWeight.SetUnit, false, 5);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("muzzle-velocity");
                attr.Value = ammo.MuzzleVelocity.ToString(ammo.MuzzleVelocity.SetUnit, false, 5);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("barrel-length");
                attr.Value = ammo.BarrelLength.ToString(ammo.BarrelLength.SetUnit, false, 5);
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("name");
                attr.Value = ammo.Name;
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("source");
                attr.Value = ammo.Source;
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("caliber");
                attr.Value = ammo.Caliber;
                node.Attributes.Append(attr);
                attr = doc.CreateAttribute("bullet-type");
                attr.Value = ammo.BulletType;
                node.Attributes.Append(attr);
                if (parent == null)
                    doc.AppendChild(node);
                else
                    parent.AppendChild(node);
            }

            public static AmmoInfo Read(XmlNode node)
            {
                AmmoInfo ammo = new AmmoInfo();
                foreach (XmlAttribute attr in node.Attributes)
                {
                    if (attr.Name == "table")
                    {
                        try
                        {
                            DragTable t = DragTableFactory.NameToDragTable(attr.Value);
                            ammo.Table = t;
                        }
                        catch (Exception )
                        {
                        }
                    }
                    else if (attr.Name == "bc")
                    {
                        double v;
                        if (Double.TryParse(attr.Value, NumberStyles.Number, CultureInfo.InvariantCulture, out v))
                            ammo.BallisticCoefficient = v;
                    }
                    else if (attr.Name == "bullet-weight")
                    {
                        Weight v;
                        Weight.Unit u;
                        if (Weight.TryParse(attr.Value, out v, out u))
                            ammo.BulletWeight = v;
                    }
                    else if (attr.Name == "muzzle-velocity")
                    {
                        Velocity v;
                        Velocity.Unit u;
                        if (Velocity.TryParse(attr.Value, out v, out u))
                            ammo.MuzzleVelocity = v;
                    }
                }
                return ammo;
            }

            public static AmmoInfoEx ReadEx(XmlNode node)
            {
                AmmoInfoEx ammo = new AmmoInfoEx();
                foreach (XmlAttribute attr in node.Attributes)
                {
                    if (attr.Name == "table")
                    {
                        try
                        {
                            DragTable t = DragTableFactory.NameToDragTable(attr.Value);
                            ammo.Table = t;
                        }
                        catch (Exception )
                        {
                        }
                    }
                    else if (attr.Name == "bc")
                    {
                        double v;
                        if (Double.TryParse(attr.Value, NumberStyles.Number, CultureInfo.InvariantCulture, out v))
                            ammo.BallisticCoefficient = v;
                    }
                    else if (attr.Name == "bullet-weight")
                    {
                        Weight v;
                        Weight.Unit u;
                        if (Weight.TryParse(attr.Value, out v, out u))
                            ammo.BulletWeight = v;
                    }
                    else if (attr.Name == "muzzle-velocity")
                    {
                        Velocity v;
                        Velocity.Unit u;
                        if (Velocity.TryParse(attr.Value, out v, out u))
                            ammo.MuzzleVelocity = v;
                    }
                    else if (attr.Name == "barrel-length")
                    {
                        Distance v;
                        Distance.Unit u;
                        if (Distance.TryParse(attr.Value, out v, out u))
                            ammo.BarrelLength = v;
                    }
                    else if (attr.Name == "name")
                    {
                        ammo.Name = attr.Value;
                    }
                    else if (attr.Name == "source")
                    {
                        ammo.Source = attr.Value;
                    }
                    else if (attr.Name == "caliber")
                    {
                        ammo.Caliber = attr.Value;
                    }
                    else if (attr.Name == "bullet-type")
                    {
                        ammo.BulletType = attr.Value;
                    }
                }
                return ammo;
            }

            public static AmmoInfo FindAndRead(XmlNode parent)
            {
                foreach (XmlNode node in parent.ChildNodes)
                    if (node.Name == "ammo-info")
                        return Read(node);
                return null;
            }

            public static AmmoInfoEx FindAndReadEx(XmlNode parent)
            {
                foreach (XmlNode node in parent.ChildNodes)
                    if (node.Name == "ammo-info-ex")
                        return ReadEx(node);
                return null;
            }

            public static void WriteFileEx(string name, AmmoInfoEx info)
            {
                XmlDocument doc = new XmlDocument();
                Write(doc, null, info);

                XmlWriterSettings settings = new XmlWriterSettings();
                settings.ConformanceLevel = ConformanceLevel.Document;
                settings.Encoding = Encoding.UTF8;
                settings.Indent = true;
                settings.IndentChars = (" ");
                settings.OmitXmlDeclaration = true;

                XmlWriter writer;
                writer = XmlWriter.Create(name, settings);
                doc.Save(writer);
                writer.Close();

            }

            public static AmmoInfoEx ReadFileEx(string name)
            {
                XmlReaderSettings settings;
                settings = new XmlReaderSettings();
                settings.ConformanceLevel = ConformanceLevel.Document;
                settings.IgnoreWhitespace = true;
                settings.IgnoreComments = true;

                XmlReader reader = XmlReader.Create(name);
                XmlDocument doc = new XmlDocument();
                doc.Load(reader);
                reader.Close();

                return ReadEx(doc.DocumentElement);
            }

            public static AmmoInfoLibrary ReadLibrary(string name)
            {
                XmlReaderSettings settings;
                settings = new XmlReaderSettings();
                settings.ConformanceLevel = ConformanceLevel.Document;
                settings.IgnoreWhitespace = true;
                settings.IgnoreComments = true;

                XmlReader reader = XmlReader.Create(name);
                XmlDocument doc = new XmlDocument();
                doc.Load(reader);
                reader.Close();

                AmmoInfoEx info;
                AmmoInfoLibrary lib = new AmmoInfoLibrary();

                for (int i = 0; i < doc.DocumentElement.ChildNodes.Count; i++)
                {
                    if (doc.DocumentElement.ChildNodes[i].Name == "ammo-info-ex")
                    {
                        info = ReadEx(doc.DocumentElement.ChildNodes[i]);
                        if (info.IsComplete())
                            lib.Add(info);
                    }
                }

                return lib;
            }
        }
    }
}