﻿using System;
using System.Globalization;
using System.Xml;
using MathEx.ExternalBallistic.Units;

namespace MathEx.ExternalBallistic
{
    public static class AtmosphereInfoController
    {
        public static class Serialization
        {
            static public void Write(XmlDocument doc, XmlNode parent, AtmosphereInfo atm)
            {
                XmlNode thisElement = doc.CreateElement("atmosphere");
                XmlAttribute attr;
                attr = doc.CreateAttribute("altitude");
                attr.Value = atm.Altitude.ToString(atm.Altitude.SetUnit, false, 8);
                thisElement.Attributes.Append(attr);
                attr = doc.CreateAttribute("pressure");
                attr.Value = atm.Pressure.ToString(atm.Pressure.SetUnit, false, 8);
                thisElement.Attributes.Append(attr);
                attr = doc.CreateAttribute("temperature");
                attr.Value = atm.Temperature.ToString(atm.Temperature.SetUnit, false, 8);
                thisElement.Attributes.Append(attr);
                attr = doc.CreateAttribute("humidity");
                attr.Value = atm.Humidity.ToString(CultureInfo.InvariantCulture);
                thisElement.Attributes.Append(attr);
                parent.AppendChild(thisElement);
            }

            static public AtmosphereInfo Read(XmlNode node)
            {
                AtmosphereInfo atm = new AtmosphereInfo();
                foreach (XmlAttribute attr in node.Attributes)
                {
                    if (attr.Name == "altitude")
                    {
                        Distance v;
                        Distance.Unit u;
                        if (Distance.TryParse(attr.Value, out v, out u))
                            atm.Altitude = v;
                    }
                    else if (attr.Name == "pressure")
                    {
                        Pressure v;
                        Pressure.Unit u;
                        if (Pressure.TryParse(attr.Value, out v, out u))
                            atm.Pressure = v;
                    }
                    else if (attr.Name == "temperature")
                    {
                        Temperature v;
                        Temperature.Unit u;
                        if (Temperature.TryParse(attr.Value, out v, out u))
                            atm.Temperature = v;
                    }
                    else if (attr.Name == "humidity")
                    {
                        double v;
                        if (Double.TryParse(attr.Value, NumberStyles.Number, CultureInfo.InvariantCulture, out v))
                            atm.Humidity = v;
                    }
                }
                return atm;
            }

            static public AtmosphereInfo FindAndRead(XmlNode parent)
            {
                foreach (XmlNode node in parent.ChildNodes)
                    if (node.Name == "atmosphere")
                        return Read(node);
                return null;
            }
        }
    }
}