﻿using System;
using System.Globalization;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using MathEx.ExternalBallistic.Units;

namespace MathEx.ExternalBallistic
{
    public enum ReportUnits
    {
        Imperial,
        Metric,
    }


    public class BallisticInfo
    {
        private Distance mRange;

        public Distance Range
        {
            get
            {
                return mRange;
            }
        }

        private Distance mPath;

        public Distance Path
        {
            get
            {
                return mPath;
            }
        }

        private Angle mHold;

        public Angle Hold
        {
            get
            {
                return mHold;
            }
        }

        private TimeSpan mTime;

        public TimeSpan Time
        {
            get
            {
                return mTime;
            }
        }

        private Distance mWindage;

        public Distance Windage
        {
            get
            {
                return mWindage;
            }
        }

        private Angle mWindageCorrection;

        public Angle WindageCorrection
        {
            get
            {
                return mWindageCorrection;
            }
        }

        private Velocity mBulletVelocity;

        public Velocity BulletVelocity
        {
            get
            {
                return mBulletVelocity;
            }
        }

        private double mMach;

        public double Mach
        {
            get
            {
                return mMach;
            }
        }

        private Energy mBulletEnergy;

        public Energy BulletEnergy
        {
            get
            {
                return mBulletEnergy;
            }
        }

        private Weight mOptimalGameWeight;

        public Weight OptimalGameWeight
        {
            get
            {
                return mOptimalGameWeight;
            }
            set
            {
                mOptimalGameWeight = value;
            }
        }


        internal BallisticInfo(Distance range, Distance path, Angle hold, TimeSpan time,
                               Distance windage, Angle windageCorrection,
                               Velocity bulletVelocity, double mach, Energy bulletEnergy, Weight ogv)
        {
            mRange = range;
            mPath = path;
            mHold = hold;
            mTime = time;
            mWindage = windage;
            mWindageCorrection = windageCorrection;
            mBulletVelocity = bulletVelocity;
            mMach = mach;
            mBulletEnergy = bulletEnergy;
            mOptimalGameWeight = ogv;
        }

        override public string ToString()
        {
            return ToString(ReportUnits.Imperial);
        }

        public string ToString(ReportUnits units)
        {
            StringBuilder b = new StringBuilder();
            switch (units)
            {
            case    ReportUnits.Imperial:
                    b.Append(mRange.ToString(Distance.Unit.Yard, true, 1));
                    b.Append(';');
                    b.Append(mBulletVelocity.ToString(Velocity.Unit.FeetPerSecond, true, 0));
                    b.Append(';');
                    b.Append(mBulletEnergy.ToString(Energy.Unit.FootPounds, true, 0));
                    b.Append(';');
                    b.Append(mPath.ToString(Distance.Unit.Inch, true, 2));
                    b.Append(';');
                    b.Append(mHold.ToString(Angle.Unit.MilDot, true, 2));
                    b.Append(';');
                    b.Append(mTime.ToString().Substring(3));
                    b.Append(';');
                    b.Append(mWindage.ToString(Distance.Unit.Inch, true, 2));
                    b.Append(';');
                    b.Append(mWindageCorrection.ToString(Angle.Unit.MilDot, true, 2));
                    break;
            case    ReportUnits.Metric:
                    b.Append(mRange.ToString(Distance.Unit.Meter, true, 1));
                    b.Append(';');
                    b.Append(mBulletVelocity.ToString(Velocity.Unit.MeterPerSecond, true, 0));
                    b.Append(';');
                    b.Append(mBulletEnergy.ToString(Energy.Unit.Joule, true, 0));
                    b.Append(';');
                    b.Append(mPath.ToString(Distance.Unit.Centimeter, true, 2));
                    b.Append(';');
                    b.Append(mHold.ToString(Angle.Unit.AMil, true, 2));
                    b.Append(';');
                    b.Append(mTime.ToString().Substring(3));
                    b.Append(';');
                    b.Append(mWindage.ToString(Distance.Unit.Centimeter, true, 2));
                    b.Append(';');
                    b.Append(mWindageCorrection.ToString(Angle.Unit.AMil, true, 2));
                    break;
            }
            return b.ToString();
        }
    }

    public class BallisticInfoCollection : IEnumerable<BallisticInfo>
    {
        private string mName;

        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                mName = value;
            }
        }

        private List<BallisticInfo> mList = new List<BallisticInfo>();

        public IEnumerator<BallisticInfo> GetEnumerator()
        {
            return mList.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return mList.GetEnumerator();
        }

        public int Count
        {
            get
            {
                return mList.Count;
            }

        }

        public BallisticInfo this[int index]
        {
            get
            {
                return mList[index];
            }

        }

        internal void Add(BallisticInfo info)
        {
            mList.Add(info);
        }

        internal void Reserve(int count)
        {
            mList.Capacity = count;
        }

        override public string ToString()
        {
            return ToString(ReportUnits.Imperial);
        }

        public string ToString(ReportUnits units)
        {
            StringBuilder b = new StringBuilder();
            b.Append("Distance;Velocity;Energy;Path;Hold;Time;Windage;Windage Adj");
            b.AppendLine();
            foreach (BallisticInfo info in mList)
            {
                b.Append(info.ToString(units));
                b.AppendLine();
            }
            return b.ToString();
        }
    }

}