﻿using System;
using System.Globalization;
using System.Text;
using MathEx.ExternalBallistic.Units;

namespace MathEx.ExternalBallistic
{
    public class ShotInfo
    {
        private string mName;

        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                mName = value;
            }
        }

        private AmmoInfo mAmmo;
        public AmmoInfo Ammo
        {
            get
            {
                return mAmmo;
            }
            set
            {
                mAmmo = value;
            }
        }

        private AtmosphereInfo mAtmosphere;
        public AtmosphereInfo Atmosphere
        {
            get
            {
                return mAtmosphere;
            }
            set
            {
                mAtmosphere = value;
            }
        }

        private WindInfo mWind;
        public WindInfo Wind
        {
            get
            {
                return mWind;
            }
            set
            {
                mWind = value;
            }
        }

        private Distance mSightHeight;
        public Distance SightHeight
        {
            get
            {
                return mSightHeight;
            }
            set
            {
                mSightHeight = value;
            }
        }

        private Distance mZeroDistance;
        public Distance ZeroDistance
        {
            get
            {
                return mZeroDistance;
            }
            set
            {
                mZeroDistance = value;
            }
        }

        private bool mNearZero;

        public bool NearZero
        {
            get
            {
                return mNearZero;
            }
            set
            {
                mNearZero = value;
            }
        }

        private Angle mShotAngle;
        public Angle ShotAngle
        {
            get
            {
                return mShotAngle;
            }
            set
            {
                mShotAngle = value;
            }
        }

        private Angle mCantAngle;
        public Angle CantAngle
        {
            get
            {
                return mShotAngle;
            }
            set
            {
                mShotAngle = value;
            }
        }

        private Angle mElevationAngle;
        public  Angle ElevationAngle
        {
            get
            {
                return mElevationAngle;
            }
            set
            {
                mElevationAngle = value;
            }
        }

        private Distance mMaxDistance = new Distance(5000, Distance.Unit.Yard);

        public Distance MaxDistance
        {
            get
            {
                return mMaxDistance;
            }
            set
            {
                mMaxDistance = value;
            }
        }

        private Distance mStep = new Distance(10, Distance.Unit.Yard);

        public Distance Step
        {
            get
            {
                return mStep;
            }
            set
            {
                mStep = value;
            }
        }

        private DriftInfo mDriftInfo;

        public DriftInfo DriftInfo
        {
            get
            {
                return mDriftInfo;
            }
            set
            {
                mDriftInfo = value;
            }
        }


        public ShotInfo()
        {
        }

        public ShotInfo(string name, AmmoInfo ammo)
        {
            mName = name;
            mAmmo = ammo;
        }

        public ShotInfo(string name, AmmoInfo ammo, Angle elevationAngle)
        {
            mName = name;
            mAmmo = ammo;
            mElevationAngle = elevationAngle;

        }

        public ShotInfo(string name, AmmoInfo ammo, AtmosphereInfo atmo, WindInfo wind, Angle elevationAngle)
        {
            mName = name;
            mAmmo = ammo;
            mWind = wind;
            mElevationAngle = elevationAngle;
            mAtmosphere = atmo;
        }

        public ShotInfo(string name, AmmoInfo ammo, AtmosphereInfo atmo, WindInfo wind, Angle elevationAngle, Angle shotAngle, Angle cantAngle, DriftInfo driftInfo)
        {
            mName = name;
            mAmmo = ammo;
            mWind = wind;
            mDriftInfo = driftInfo;
            mShotAngle = shotAngle;
            mCantAngle = cantAngle;
            mElevationAngle = elevationAngle;
            mAtmosphere = atmo;
        }

        override public string ToString()
        {
            StringBuilder b = new StringBuilder();
            if (Name != null)
            {
                b.Append("Name=");
                b.Append(Name);
                b.AppendLine();
            }
            if (Ammo != null)
            {
                b.Append("Ammo:");
                b.AppendLine();
                b.Append(Ammo.ToString());
            }
            if (Atmosphere != null)
            {
                b.Append("Atmosphere:");
                b.AppendLine();
                b.Append(Atmosphere.ToString());
            }

            if (Wind != null)
            {
                b.Append("Wind:");
                b.AppendLine();
                b.Append(Wind.ToString());
            }

            if (mShotAngle != null)
            {
                b.Append("Shooting Angle=");
                b.Append(mShotAngle.ToString(mShotAngle.SetUnit));
                b.AppendLine();
            }

            if (mCantAngle != null)
            {
                b.Append("Shooting Angle=");
                b.Append(mCantAngle.ToString(mCantAngle.SetUnit));
                b.AppendLine();
            }

            if (mElevationAngle != null)
            {
                b.Append("Elevation Angle=");
                b.Append(mElevationAngle.ToString(mElevationAngle.SetUnit));
                b.AppendLine();
            }

            if (mMaxDistance != null)
            {
                b.Append("Max distance=");
                b.Append(mMaxDistance.ToString(mMaxDistance.SetUnit));
                b.AppendLine();
            }
            if (mStep != null)
            {
                b.Append("Step=");
                b.Append(mStep.ToString(mStep.SetUnit));
                b.AppendLine();
            }
            return b.ToString();
        }
    }
}