﻿using System;
using System.Globalization;
using System.Xml;

namespace MathEx.ExternalBallistic.Units
{

    public static class UnitSerialization
    {
        public static void WriteAngle(XmlDocument doc, XmlNode node, string attribute, Angle v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Angle ReadAngle(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Angle v;
                    Angle.Unit u;
                    if (Angle.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteDistance(XmlDocument doc, XmlNode node, string attribute, Distance v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Distance ReadDistance(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Distance v;
                    Distance.Unit u;
                    if (Distance.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteEnergy(XmlDocument doc, XmlNode node, string attribute, Energy v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Energy ReadEnergy(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Energy v;
                    Energy.Unit u;
                    if (Energy.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WritePressure(XmlDocument doc, XmlNode node, string attribute, Pressure v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Pressure ReadPressure(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Pressure v;
                    Pressure.Unit u;
                    if (Pressure.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteTemperature(XmlDocument doc, XmlNode node, string attribute, Temperature v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Temperature ReadTemperature(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Temperature v;
                    Temperature.Unit u;
                    if (Temperature.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteVelocity(XmlDocument doc, XmlNode node, string attribute, Velocity v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Velocity ReadVelocity(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Velocity v;
                    Velocity.Unit u;
                    if (Velocity.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteWeight(XmlDocument doc, XmlNode node, string attribute, Weight v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(v.SetUnit, false, 8);
            node.Attributes.Append(attr);
        }

        public static Weight ReadWeight(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    Weight v;
                    Weight.Unit u;
                    if (Weight.TryParse(attr.Value, out v, out u))
                        return v;
                    else
                        return null;

                }
            }
            return null;
        }

        public static void WriteDouble(XmlDocument doc, XmlNode node, string attribute, double v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString("0.000", CultureInfo.InvariantCulture);
            node.Attributes.Append(attr);
        }

        public static double ReadDouble(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    double v;
                    if (Double.TryParse(attr.Value, out v))
                        return v;
                    else
                        return 0;

                }
            }
            return 0;
        }

        public static void WriteInt(XmlDocument doc, XmlNode node, string attribute, int v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v.ToString(CultureInfo.InvariantCulture);
            node.Attributes.Append(attr);
        }

        public static int ReadInt(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
            {
                if (attr.Name == attribute)
                {
                    int v;
                    if (Int32.TryParse(attr.Value, out v))
                        return v;
                    else
                        return 0;

                }
            }
            return 0;
        }

        public static void WriteBool(XmlDocument doc, XmlNode node, string attribute, bool v)
        {
            XmlAttribute attr = doc.CreateAttribute(attribute);
            attr.Value = v ? "true" : "false";
            node.Attributes.Append(attr);
        }

        public static bool ReadBool(XmlNode node, string attribute)
        {
            foreach (XmlAttribute attr in node.Attributes)
                if (attr.Name == attribute)
                    return attr.Value == "true";
            return false;
        }

    }

}