﻿using System;
using System.IO;
using System.Globalization;
using System.Text;
using System.Xml;
using MathEx.ExternalBallistic.Units;

namespace MathEx.ExternalBallistic
{
    public static class WindInfoController
    {
        public static class Serialization
        {
            public static void Write(XmlDocument doc, XmlNode parent, WindInfo windInfo)
            {
                XmlNode zero = doc.CreateElement("wind-info");
                XmlAttribute attr;
                attr = doc.CreateAttribute("speed");
                attr.Value = windInfo.Speed.ToString(windInfo.Speed.SetUnit, false, 8);
                zero.Attributes.Append(attr);
                attr = doc.CreateAttribute("direction");
                attr.Value = windInfo.Direction.ToString(windInfo.Direction.SetUnit, false, 8);
                zero.Attributes.Append(attr);
                if (parent == null)
                    doc.AppendChild(zero);
                else
                    parent.AppendChild(zero);
            }

            public static WindInfo Read(XmlNode node)
            {
                WindInfo windInfo = new WindInfo();

                foreach (XmlAttribute attr in node.Attributes)
                {
                    if (attr.Name == "speed")
                    {
                        Velocity v;
                        Velocity.Unit u;
                        if (Velocity.TryParse(attr.Value, out v, out u))
                            windInfo.Speed = v;
                    }
                    if (attr.Name == "direction")
                    {
                        Angle v;
                        Angle.Unit u;
                        if (Angle.TryParse(attr.Value, out v, out u))
                            windInfo.Direction = v;
                    }
                }
                return windInfo;
            }

            public static WindInfo FindAndRead(XmlNode parent)
            {
                foreach (XmlNode node in parent.ChildNodes)
                    if (node.Name == "wind-info")
                        return Read(node);
                return null;
            }

        }
    }
}