﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using MathEx.ExternalBallistic;
using MathEx.ExternalBallistic.Units;

namespace Gehtsoft.BallisticCalculator.UI
{
    public partial class SpinDriftInfoInput : UserControl
    {
        public SpinDriftInfoInput()
        {
            InitializeComponent();
            DriftInfoPresetList.FillComboBox(comboBoxPreset);
            comboBoxPreset.SelectedIndex = -1;
            radioButtonRH.Checked = true;
        }

        private void comboBoxPreset_SelectedIndexChanged(object sender, EventArgs e)
        {
            DriftInfoPreset preset = comboBoxPreset.SelectedItem as DriftInfoPreset;
            if (preset != null)
            {
                distanceDiameter.Value = preset.BulletDiameter;
                distanceLength.Value = preset.BulletLength;
                distanceRifling.Value = preset.RiflingTwist;
                radioButtonLH.Checked = !preset.RightHandTwist;
                radioButtonRH.Checked = preset.RightHandTwist;
            }
        }

        public DriftInfo GetDriftInfo()
        {
            if (!checkBoxCalculate.Checked)
                return null;

            return new DriftInfo(distanceLength.Value, distanceDiameter.Value, distanceRifling.Value, radioButtonRH.Checked);
        }

        public void AddControlsToFormState(FormState state)
        {
            AddControlsToFormState(state, "");
        }

        public void AddControlsToFormState(FormState state, string prefix)
        {
            state.AddControl(prefix + "drift-diameter", distanceDiameter);
            state.AddControl(prefix + "drift-length", distanceLength);
            state.AddControl(prefix + "drift-step", distanceRifling);
            state.AddControl(prefix + "drift-lh", radioButtonLH);
            state.AddControl(prefix + "drift-rh", radioButtonRH);
        }

        
    }
}
