﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using MathEx.ExternalBallistic.Units;

namespace Gehtsoft.BallisticCalculator.UI
{
    class DriftInfoPreset
    {
        string mName;
        Distance mBulletDiameter;
        Distance mBulletLength;
        Distance mRiflingTwist;
        bool mRightHand;

        public Distance BulletDiameter
        {
            get
            {
                return mBulletDiameter;
            }
        }

        public Distance BulletLength
        {
            get
            {
                return mBulletLength;
            }
        }

        public Distance RiflingTwist
        {
            get
            {
                return mRiflingTwist;
            }
        }

        public bool RightHandTwist
        {
            get
            {
                return mRightHand;
            }
        }

        internal DriftInfoPreset(string name, Distance bulletDiameter, Distance bulletLength, Distance riflingTwist, bool rightHand)
        {
            mName = name;
            mBulletDiameter = bulletDiameter;
            mBulletLength = bulletLength;
            mRiflingTwist = riflingTwist;
            mRightHand = true;
        }

        override public string ToString()
        {
            return mName;
        }
    }

    public static class DriftInfoPresetList
    {
        static DriftInfoPreset[] mDriftPresets = new DriftInfoPreset[]{
            new DriftInfoPreset(".22 LR Match", new Distance(0.223, Distance.Unit.Inch), new Distance(0.56, Distance.Unit.Inch), new Distance(9, Distance.Unit.Inch), true),
            new DriftInfoPreset(".22 LR 10/22", new Distance(0.223, Distance.Unit.Inch), new Distance(0.56, Distance.Unit.Inch), new Distance(16, Distance.Unit.Inch), true),
            new DriftInfoPreset("5.45x39", new Distance(5.62, Distance.Unit.Millimeter), new Distance(25.5, Distance.Unit.Millimeter), new Distance(196, Distance.Unit.Millimeter), true),
            new DriftInfoPreset("5.56x45 M16A1", new Distance(0.224, Distance.Unit.Inch), new Distance(0.9, Distance.Unit.Inch), new Distance(12, Distance.Unit.Inch), true),
            new DriftInfoPreset("5.56x45 M16A3", new Distance(0.224, Distance.Unit.Inch), new Distance(0.9, Distance.Unit.Inch), new Distance(7, Distance.Unit.Inch), true),
            new DriftInfoPreset("300 AAC", new Distance(0.308, Distance.Unit.Inch), new Distance(1.3, Distance.Unit.Inch), new Distance(7, Distance.Unit.Inch), true),
            new DriftInfoPreset("7.62x39", new Distance(7.85, Distance.Unit.Millimeter), new Distance(26, Distance.Unit.Millimeter), new Distance(240, Distance.Unit.Millimeter), true),
            new DriftInfoPreset(".308", new Distance(0.308, Distance.Unit.Inch), new Distance(1.3, Distance.Unit.Inch), new Distance(12, Distance.Unit.Inch), true),
            new DriftInfoPreset("7.62x54R", new Distance(7.92, Distance.Unit.Millimeter), new Distance(32, Distance.Unit.Millimeter), new Distance(240, Distance.Unit.Millimeter), true),
            new DriftInfoPreset("9x39", new Distance(9.3, Distance.Unit.Millimeter), new Distance(35, Distance.Unit.Millimeter), new Distance(420, Distance.Unit.Millimeter), true),
            new DriftInfoPreset(".338 Lapua", new Distance(0.338, Distance.Unit.Inch), new Distance(1.5, Distance.Unit.Inch), new Distance(10, Distance.Unit.Inch), true),
            new DriftInfoPreset("50BMG", new Distance(0.51, Distance.Unit.Inch), new Distance(2.3, Distance.Unit.Inch), new Distance(15, Distance.Unit.Inch), true),
        };

        public static void FillComboBox(ComboBox box)
        {
            foreach (DriftInfoPreset preset in mDriftPresets)
                box.Items.Add(preset);
        }
    }
}
