﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using MathEx.ExternalBallistic;
using MathEx.ExternalBallistic.Units;

namespace Gehtsoft.BallisticCalculator.UI
{
    public partial class ZeroInfoInput : UserControl, IMeasurementSystemListener
    {
        public ZeroInfoInput()
        {
            InitializeComponent();
            ZeroPresetController.FillComboBox(comboBoxPreset);
            bulletInfoInput.AdvancedMode = false;
        }

        private void comboBoxPreset_SelectedIndexChanged(object sender, EventArgs e)
        {
            ZeroPreset preset = comboBoxPreset.SelectedItem as ZeroPreset;
            if (preset != null)
            {
                distanceSight.Value = preset.ZeroHeight;
                distanceZero.Value = preset.ZeroDistance;
                checkBoxNearZero.Checked = preset.NearZero;
            }
        }

        private void checkBoxOtherAmmo_CheckedChanged(object sender, EventArgs e)
        {
            bulletInfoInput.Enabled = checkBoxOtherAmmo.Checked;
        }

        private void checkBoxOtherAtmo_CheckedChanged(object sender, EventArgs e)
        {
            atmoInfoInput.Enabled = checkBoxOtherAtmo.Checked;
        }

        private MeasurementSystem mMeasurementSystem;

        public MeasurementSystem MeasurementSystem
        {
            get
            {
                return mMeasurementSystem;
            }
            set
            {
                mMeasurementSystem = value;
                bulletInfoInput.MeasurementSystem = value;
                atmoInfoInput.MeasurementSystem = value;
                if (mMeasurementSystem == UI.MeasurementSystem.Imperial)
                {
                    distanceSight.Unit = Distance.Unit.Inch;
                    distanceZero.Unit = Distance.Unit.Yard;
                }
                else
                {
                    distanceSight.Unit = Distance.Unit.Centimeter;
                    distanceZero.Unit = Distance.Unit.Meter;
                }
            }
        }

        public void FillShotInfo(ShotInfo shotInfo)
        {
            shotInfo.ZeroDistance = distanceZero.Value;
            shotInfo.NearZero = checkBoxNearZero.Checked;
            shotInfo.SightHeight = distanceSight.Value;
            if (checkBoxOtherAmmo.Checked)
                shotInfo.Ammo = bulletInfoInput.GetAmmoInfo();
            if (checkBoxOtherAtmo.Checked)
                shotInfo.Atmosphere = atmoInfoInput.GetAtmosphere();
        }

        public void Initialize(MeasurementSystem msi)
        {
            MeasurementSystem = msi;
            distanceSight.Value = msi == UI.MeasurementSystem.Imperial ? new Distance(1.5, Distance.Unit.Inch) : new Distance(5, Distance.Unit.Centimeter);
            distanceZero.Value = msi == UI.MeasurementSystem.Imperial ? new Distance(100, Distance.Unit.Yard) : new Distance(100, Distance.Unit.Meter);
        }

        public void AddControlsToFormState(FormState state)
        {
            AddControlsToFormState(state, "");
        }

        public void AddControlsToFormState(FormState state, string prefix)
        {
            state.AddControl("" + "zero-sight-height", distanceSight);
            state.AddControl("" + "zero-distance", distanceZero);
            state.AddControl("" + "zero-other-bullet", checkBoxOtherAmmo);
            state.AddControl("" + "zero-other-atmo", checkBoxOtherAtmo);
            bulletInfoInput.AddControlsToFormState(state, "zero-");
            atmoInfoInput.AddControlsToFormState(state, "zero-");
        }
    }
}
