﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using MathEx.ExternalBallistic.Units;

namespace Gehtsoft.BallisticCalculator.UI
{
    public class FormState
    {
        private XmlDocument mDocument;

        class Pair
        {
            private string name;
            private Control control;

            internal string Name
            {
                get
                {
                    return name;
                }
            }

            internal Control Control
            {
                get
                {
                    return control;
                }
            }

            internal Pair(string n, Control c)
            {
                name = n;
                control = c;
            }
        }

        List<Pair> mPairs = new List<Pair>();

        public FormState()
        {
        }

        public void AddControl(string name, Control control)
        {
            Pair pair = new Pair(name, control);
            mPairs.Add(pair);
        }

        public void GatherFrom()
        {
            XmlDocument doc = new XmlDocument();
            XmlNode parent = doc.CreateElement("form");
            XmlNode prop;
            XmlAttribute value;

            doc.AppendChild(parent);
            foreach (Pair p in mPairs)
            {
                if (p.Control is CheckBox)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CheckBox).Checked ? "true" : "false";
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                if (p.Control is RadioButton)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as RadioButton).Checked ? "true" : "false";
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is TextBox)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as TextBox).Text;
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is ComboBox)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as ComboBox).Text;
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is NumericUpDown)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as NumericUpDown).Value.ToString();
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is AngleSelector)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as AngleSelector).Angle.ToString();
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomAngleControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomAngleControl).Value.ToString((p.Control as CustomAngleControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomDistanceControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomDistanceControl).Value.ToString((p.Control as CustomDistanceControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomPressureControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomPressureControl).Value.ToString((p.Control as CustomPressureControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomTemperatureControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomTemperatureControl).Value.ToString((p.Control as CustomTemperatureControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomVelocityControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomVelocityControl).Value.ToString((p.Control as CustomVelocityControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is CustomWeightControl)
                {
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("value");
                    value.Value = (p.Control as CustomWeightControl).Value.ToString((p.Control as CustomWeightControl).Value.SetUnit, false, 8);
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is BallisticTable)
                {
                    BallisticTable t = p.Control as BallisticTable;
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("measurement-system");
                    value.Value = ((int)t.MeasurementSystem).ToString();
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                    value = doc.CreateAttribute("angle-unit");
                    value.Value = ((int)t.AngleUnits).ToString();
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }
                else if (p.Control is BallisticGraphControl)
                {
                    BallisticGraphControl t = p.Control as BallisticGraphControl;
                    prop = doc.CreateElement(p.Name);
                    value = doc.CreateAttribute("data-displayed");
                    value.Value = ((int)t.GraphData).ToString();
                    prop.Attributes.Append(value);
                    parent.AppendChild(prop);
                }

                mDocument = doc;
            }
        }

        public void Save(string name)
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.ConformanceLevel = ConformanceLevel.Document;
            settings.Encoding = Encoding.UTF8;
            settings.Indent = true;
            settings.IndentChars = (" ");
            settings.OmitXmlDeclaration = true;

            using (XmlWriter writer = XmlWriter.Create(name, settings))
            {
                mDocument.Save(writer);
                writer.Close();
            }
        }

        public void FillForm()
        {
            foreach (XmlNode node in mDocument.DocumentElement)
            {
                for (int i = 0; i < mPairs.Count; i++)
                    if (node.Name == mPairs[i].Name)
                    {
                        Pair p = mPairs[i];
                        if (p.Control is CheckBox)
                        {
                            if (node.Attributes["value"] != null)
                                (p.Control as CheckBox).Checked = node.Attributes["value"].Value == "true";
                        }
                        if (p.Control is RadioButton)
                        {
                            if (node.Attributes["value"] != null)
                                (p.Control as RadioButton).Checked = node.Attributes["value"].Value == "true";
                        }
                        else if (p.Control is TextBox)
                        {
                            if (node.Attributes["value"] != null)
                                (p.Control as TextBox).Text = node.Attributes["value"].Value;
                        }
                        else if (p.Control is ComboBox)
                        {
                            if (node.Attributes["value"] != null)
                                (p.Control as ComboBox).Text = node.Attributes["value"].Value;
                        }
                        else if (p.Control is NumericUpDown)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                decimal v;
                                if (Decimal.TryParse(node.Attributes["value"].Value, out v))
                                    (p.Control as NumericUpDown).Value = v;
                            }
                        }
                        else if (p.Control is AngleSelector)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                int v;
                                if (Int32.TryParse(node.Attributes["value"].Value, out v))
                                    (p.Control as AngleSelector).Angle = v;
                            }
                        }
                        else if (p.Control is CustomAngleControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Angle v;
                                Angle.Unit u;
                                if (Angle.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomAngleControl).Value = v;
                            }
                        }
                        else if (p.Control is CustomDistanceControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Distance v;
                                Distance.Unit u;
                                if (Distance.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomDistanceControl).Value = v;
                            }
                        }
                        else if (p.Control is CustomPressureControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Pressure v;
                                Pressure.Unit u;
                                if (Pressure.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomPressureControl).Value = v;
                            }
                        }
                        else if (p.Control is CustomTemperatureControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Temperature v;
                                Temperature.Unit u;
                                if (Temperature.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomTemperatureControl).Value = v;
                            }
                        }
                        else if (p.Control is CustomVelocityControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Velocity v;
                                Velocity.Unit u;
                                if (Velocity.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomVelocityControl).Value = v;
                            }
                        }
                        else if (p.Control is CustomWeightControl)
                        {
                            if (node.Attributes["value"] != null)
                            {
                                Weight v;
                                Weight.Unit u;
                                if (Weight.TryParse(node.Attributes["value"].Value, out v, out u))
                                    (p.Control as CustomWeightControl).Value = v;
                            }
                        }
                        else if (p.Control is BallisticTable)
                        {
                            int x = 0;
                            BallisticTable t = p.Control as BallisticTable;
                            if (node.Attributes["measurement-system"] != null)
                            {
                                if (Int32.TryParse(node.Attributes["measurement-system"].Value, out x))
                                    t.MeasurementSystem = (MeasurementSystem)x;
                            }
                            if (node.Attributes["angle-unit"] != null)
                            {
                                if (Int32.TryParse(node.Attributes["angle-unit"].Value, out x))
                                    t.AngleUnits = (Angle.Unit)x;
                            }

                        }
                        else if (p.Control is BallisticGraphControl)
                        {
                            int x = 0;
                            BallisticGraphControl t = p.Control as BallisticGraphControl;

                            if (node.Attributes["data-displayed"] != null)
                            {
                                if (Int32.TryParse(node.Attributes["data-displayed"].Value, out x))
                                    t.GraphData = (ColumnData)x;
                            }
                        }
                    }
            }
        }

        public void Load(string filename)
        {
            XmlReaderSettings settings;
            settings = new XmlReaderSettings();
            settings.ConformanceLevel = ConformanceLevel.Document;
            settings.IgnoreWhitespace = true;
            settings.IgnoreComments = true;

            XmlDocument doc = new XmlDocument();
            using (XmlReader reader = XmlReader.Create(filename))
            {
                doc.Load(reader);
                reader.Close();
            }
            mDocument = doc;
        }
    }
}
